@@ -1,10 +1,15 @@
 
 ------------------------------------------
--- RACER 0.6
+-- RACER 0.8
 -- map-independant racing script
 -- by mikade
 -----------------------------------------
 
+-- DEVELOPER WARNING - FOR OFFICIAL DEVELOPMENT --
+-- Be careful when editig this script, do not introduce changes lightly!
+-- This script is used for time records on the official Hedgewars server.
+-- Introducing breaking changes means we have to invalidate past time records!
+
 -----------------------------------
 --0.1: took all the code from crazy racer and scrapped most of it
 -----------------------------------
@@ -73,6 +78,11 @@
 
 -- switch to first available weapon if starting race with no weapon selected
 
+-------
+-- 0.8
+-------
+-- allow different boost directions
+
 -----------------------------
 -- SCRIPT BEGINS
 -----------------------------
@@ -94,7 +104,7 @@ local fastX = {}
 local fastY = {}
 local fastCount = 0
 local fastIndex = 0
-local fastColour
+local fastColour = 0xffffffff
 
 local currX = {}
 local currY = {}
@@ -128,8 +138,8 @@ local teamScore = {}
 
 local cGear = nil
 
-local bestClan = nil
-local bestTime = nil
+local bestClan = 10
+local bestTime = 1000000
 
 local gameBegun = false
 local gameOver = false
@@ -141,7 +151,7 @@ local wpX = {}
 local wpY = {}
 local wpCol = {}
 local wpActive = {}
-local wpRad = 450 --75
+local wpRad = 450
 local wpCount = 0
 local wpLimit = 8
 
@@ -150,16 +160,56 @@ local usedWeapons = {}
 local roundN
 local lastRound
 local RoundHasChanged
+local turnSkipped = false
+
+local boostX = 0
+local boostY = 0
+local boostValue = 1
 
 -------------------
 -- general methods
 -------------------
 
+--[[
+Parameters syntax:
+
+teamrope=true
+--> The team will be colored in the color of the team.
+
+rounds=N
+--> The game will be played with N rounds (default: 3)
+
+waypointradius=N
+--> The waypoints have a radius of N pixels (default: 450)
+
+maxwaypoints=N
+--> The maximum number of waypoints to be placed (default: 8)
+
+]]
+
 function onParameters()
     parseParams()
     if params["teamrope"] ~= nil then
         TeamRope = true
     end
+    if params["rounds"] ~= nil then
+        roundLimit = math.max(1, math.floor(tonumber(params["rounds"])))
+        if type(roundLimit) ~= "number" then
+             roundLimit = 3
+        end
+    end
+    if params["waypointradius"] ~= nil then
+        wpRad = math.max(40, math.floor(tonumber(params["waypointradius"])))
+        if type(wpRad) ~= "number" then
+             wpRad = 450
+        end
+    end
+    if params["maxwaypoints"] ~= nil then
+        wpLimit = math.max(2, math.floor(tonumber(params["maxwaypoints"])))
+        if type(wpLimit) ~= "number" then
+             wpLimit = 8
+        end
+    end
 end
 
 function RebuildTeamInfo()
@@ -167,10 +217,10 @@ function RebuildTeamInfo()
 
         -- make a list of individual team names
         for i = 0, (TeamsCount-1) do
-                teamNameArr[i] = " " -- = i
+                teamNameArr[i] = " "
                 teamSize[i] = 0
                 teamIndex[i] = 0
-                teamScore[i] = 100000
+                teamScore[i] = 1000000
         end
         numTeams = 0
 
@@ -224,6 +274,31 @@ end
 -- RACER METHODS
 -----------------
 
+function onLeft()
+	boostX = boostX +boostValue
+end
+function onLeftUp()
+	boostX = boostX -boostValue
+end
+function onRight()
+	boostX = boostX -boostValue
+end
+function onRightUp()
+	boostX = boostX +boostValue
+end
+function onUp()
+	boostY = boostY +boostValue
+end
+function onUpUp()
+	boostY = boostY -boostValue
+end
+function onDown()
+	boostY = boostY -boostValue
+end
+function onDownUp()
+	boostY = boostY +boostValue
+end
+
 function CheckWaypoints()
 
         trackFinished = true
@@ -237,17 +312,10 @@ function CheckWaypoints()
                 g1Y = g1Y - g2Y
                 dist = (g1X*g1X) + (g1Y*g1Y)
 
-                --if i == 0 then
-                --      AddCaption(dist .. "/" .. (wpRad*wpRad) )
-                --end
-
                 NR = (48/100*wpRad)/2
 
                 if dist < (NR*NR) then
-                --if dist < (wpRad*wpRad) then
-                        --AddCaption("howdy")
-                        wpActive[i] = true
-                        wpCol[i] = GetClanColor(GetHogClan(CurrentHedgehog)) -- new                             --GetClanColor(1)
+                        wpCol[i] = GetClanColor(GetHogClan(CurrentHedgehog))
                         SetVisualGearValues(wpCirc[i], wpX[i], wpY[i], 20, 100, 1, 10, 0, wpRad, 5, wpCol[i])
 
                         wpRem = 0
@@ -257,7 +325,16 @@ function CheckWaypoints()
                                 end
                         end
 
-                        AddCaption(loc("Way-Points Remaining") .. ": " .. wpRem,0xffba00ff,capgrpAmmoinfo)
+                        if wpActive[i] == false then
+                                local wpMessage = ""
+                                if wpRem-1 == 0 then
+                                         wpMessage = loc("Track completed!")
+                                else
+                                         wpMessage = string.format(loc("Waypoints remaining: %d"), wpRem-1)
+                                end
+                                AddCaption(wpMessage, 0xffba00ff, capgrpGameState)
+                        end
+                        wpActive[i] = true
 
                 end
 
@@ -272,12 +349,7 @@ function CheckWaypoints()
 end
 
 function AdjustScores()
-
-        if bestTime == nil then
-                bestTime = 100000
-                bestClan = 10
-                bestTimeComment = "N/A"
-        end
+	bestTimeComment = loc("Did not finish")
 
         newScore = false
 
@@ -293,9 +365,6 @@ function AdjustScores()
                 end
         end
 
-        --bestTime = 100000
-        --bestClan = 10
-
         -- find the best time out of those so far
         for i = 0, (numTeams-1) do
                 if teamScore[i] < bestTime then
@@ -304,38 +373,33 @@ function AdjustScores()
                 end
         end
 
-        if bestTime ~= 100000 then
-                bestTimeComment = (bestTime/1000) ..loc("s")
+        if bestTime ~= 1000000 then
+                bestTimeComment = string.format(loc("%.1fs"), (bestTime/1000))
         end
 
         if newScore == true then
                 if trackTime == bestTime then -- best time of the race
-                        ShowMission(loc("RACER"),
-                        loc("TRACK COMPLETED"),
-                        loc("NEW RACE RECORD: ") .. (trackTime/1000) ..loc("s") .. "|" ..
-                        loc("WINNING TIME: ") .. bestTimeComment, 0, 4000)
+                        ShowMission(loc("Racer"),
+                        loc("Track completed!"),
+                        string.format(loc("New race record: %.1fs"), (trackTime/1000)) .. "|" ..
+                        string.format(loc("Winning time: %s"), bestTimeComment), 0, 4000)
                         PlaySound(sndHomerun)
                 else    -- best time for the clan
-                        ShowMission(loc("RACER"),
-                        loc("TRACK COMPLETED"),
-                        loc("NEW CLAN RECORD: ") .. (trackTime/1000) ..loc("s") .. "|" ..
-                        loc("WINNING TIME: ") .. bestTimeComment, 4, 4000)
+                        ShowMission(loc("Racer"),
+                        loc("Track completed!"),
+                        string.format(loc("New clan record: %.1fs"), (trackTime/1000)) .. "|" ..
+                        string.format(loc("Winning time: %s"), bestTimeComment), 4, 4000)
                 end
         else -- not any kind of new score
-                ShowMission(loc("RACER"),
-                loc("TRACK COMPLETED"),
-                loc("TIME: ") .. (trackTime/1000) ..loc("s") .. "|" ..
-                loc("WINNING TIME: ") .. bestTimeComment, -amSkip, 4000)
+                ShowMission(loc("Racer"),
+                loc("Track completed!"),
+                string.format(loc("Time: %.1fs"), (trackTime/1000)) .. "|" ..
+                string.format(loc("Winning time: %s"), bestTimeComment), -amSkip, 4000)
                 PlaySound(sndHellish)
         end
 
 
-        --------
-        --new
-        --------
-
         if bestTime == trackTime then
-                --AddCaption("wooooooooooooooooooooooooooooo")
 
                 fastColour = GetClanColor(GetHogClan(CurrentHedgehog))
 
@@ -347,8 +411,6 @@ function AdjustScores()
                 fastCount = currCount
                 fastIndex = 0
 
-                --currCount = 0 -- is this needed?
-
         else
                 currCount = 0
                 fastIndex = 0
@@ -363,93 +425,94 @@ function onNewRound()
 
         totalComment = ""
         for i = 0, (TeamsCount-1) do
-                        if teamNameArr[i] ~= " " then                           -- teamScore[teamClan[i]]
-                                teamComment[i] = teamNameArr[i] .. ": " .. (teamScore[i]/1000) .. loc("s|")
-                                totalComment = totalComment .. teamComment[i]
-                        elseif teamNameArr[i] == " " then
-                                teamComment[i] = "|"
+                        if teamNameArr[i] ~= " " and teamScore[i] ~= 1000000 then
+                                teamComment[i] = string.format(loc("%s: %.1fs"), teamNameArr[i], (teamScore[i]/1000)) .. "|"
+                        else
+                                teamComment[i] = string.format(loc("%s: Did not finish"), teamNameArr[i]) .. "|"
                         end
+                        totalComment = totalComment .. teamComment[i]
         end
 
-        ShowMission(    loc("RACER"),
-                                        loc("STATUS UPDATE"),
-                                        loc("Rounds Complete: ") .. roundNumber .. "/" .. roundLimit .. "|" .. " " .. "|" ..
-                                        loc("Best Team Times: ") .. "|" .. totalComment, 0, 4000)
+        ShowMission(    loc("Racer"),
+                                        loc("Status update"),
+                                        string.format(loc("Rounds complete: %d/%d"), roundNumber, roundLimit) .. "|" .. " " .. "|" ..
+                                        loc("Best team times: ") .. "|" .. totalComment, 0, 4000)
 
         -- end game if its at round limit
         if roundNumber >= roundLimit then
-                for i = 0, (numhhs-1) do
-                        if GetHogClan(hhs[i]) ~= bestClan then
-                                SetEffect(hhs[i], heResurrectable, 0)
-                                SetHealth(hhs[i],0)
+                -- Sort the scores for the ranking list
+                local unfinishedArray = {}
+                local sortedTeams = {}
+                local k = 1
+                for i = 0, TeamsCount-1 do
+                        if teamScore[i] ~= 1000000 and teamNameArr[i] ~= " " then
+                               sortedTeams[k] = {}
+                               sortedTeams[k].name = teamNameArr[i]
+                               sortedTeams[k].score = teamScore[i]
+                               k = k + 1
+                        else
+                               table.insert(unfinishedArray, string.format(loc("%s did not finish the race."), teamNameArr[i]))
                         end
                 end
-                gameOver = true
-                TurnTimeLeft = 1
+                table.sort(sortedTeams, function(team1, team2) return team1.score < team2.score end)
+
+                -- Write all the stats!
+
+                for i = 1, #sortedTeams do
+                        SendStat(siPointType, loc("milliseconds"))
+                        SendStat(siPlayerKills, sortedTeams[i].score, sortedTeams[i].name)
+                end
+
+                if #sortedTeams >= 1 then
+                        SendStat(siGameResult, string.format(loc("%s wins!"), sortedTeams[1].name))
+                        SendStat(siGameResult, string.format(loc("%s wins!"), sortedTeams[1].name))
+                        SendStat(siCustomAchievement, string.format(loc("%s wins with a best time of %.1fs."), sortedTeams[1].name, (sortedTeams[1].score/1000)))
+                        for i=1,#unfinishedArray do
+                                 SendStat(siCustomAchievement, unfinishedArray[i])
+                        end
+                else
+                        SendStat(siGameResult, loc("Round draw"))
+                        SendStat(siCustomAchievement, loc("Nobody managed to finish the race. What a shame!"))
+                        if specialPointsCount > 0 then
+                                SendStat(siCustomAchievement, loc("Maybe you should try an easier map next time."))
+                        else
+                                SendStat(siCustomAchievement, loc("Maybe you should try easier waypoints next time."))
+                        end
+                end
+
+		-- Kill all the losers
+		for i = 0, (numhhs-1) do
+			if GetHogClan(hhs[i]) ~= bestClan then
+				SetEffect(hhs[i], heResurrectable, 0)
+				SetHealth(hhs[i],0)
+			end
+		end
+
+		gameOver = true
+		EndTurn(true)
+
         end
 
 end
 
 function CheckForNewRound()
 
-        -------------
-        ------ new
-        -------------
-
-        --[[turnN = turnN + 1
-        if gameBegun == false then
-                if turnN == 2 then
-                        for i = 0, (numhhs-1) do
-                                if hhs[i] ~= nil then
-                                        SetEffect(hhs[i], heResurrectable, 0)
-                                        SetHealth(hhs[i],0)
-                                end
-                        end
-                        gameOver = true
-                        TurnTimeLeft = 1
-                end
-        else
-
-
-        end]]
-
-        --[[if roundBegun == true then
-
-                if RoundHasChanged == true then
-                        roundN = roundN + 1
-                        RoundHasChanged = false
-                        onNewRound()
-                end
-
-                if lastRound ~= TotalRounds then -- new round, but not really
-
-                        if RoundHasChanged == false then
-                                RoundHasChanged = true
-                        end
-
-                end
-
-                AddCaption("RoundN:" .. roundN .. "; " .. "TR: " .. TotalRounds)
-
-                lastRound = TotalRounds
-
-        end]]
-
-        ------------
-        ----- old
-        ------------
-
         if GetHogClan(CurrentHedgehog) == firstClan then
                 onNewRound()
         end
 
 end
 
-function DisableTumbler()
-        currCount = 0
-        fastIndex = 0
-        TurnTimeLeft = 0
-        racerActive = false -- newadd
+function DisableTumbler(endTurn)
+        if endTurn == nil then endTurn = true end
+        if racerActive then
+                currCount = 0
+                fastIndex = 0
+                if endTurn then
+                        EndTurn(true)
+                end
+                racerActive = false -- newadd
+        end
 end
 
 function HandleGhost()
@@ -468,16 +531,8 @@ function HandleGhost()
                 g1, g2, g3, g4, g5, g6, g7, g8, g9, g10 = GetVisualGearValues(tempE)
                 SetVisualGearValues(tempE, g1, g2, g3, g4, g5, g6, g7, g8, g9, fastColour )
 
-                --AddCaption("fC: " .. fastIndex .. " / " .. fastCount)
-
-        else
-
-                --AddCaption("excep fC: " .. fastIndex .. " / " .. fastCount)
-
         end
 
-
-
 end
 
 function TryRepositionHogs()
@@ -503,57 +558,121 @@ function onGameInit()
         CaseFreq = 0
         TurnTime = 90000
         WaterRise = 0
+        HealthDecrease = 0
+
 end
 
+function InstructionsBuild()
+        ShowMission(
+                loc("Racer"),
+                loc("A Hedgewars mini-game"),
+                loc("Build a track and race.") .. "|" ..
+                string.format(loc("Round limit: %d"), roundLimit),
+                4, 4000)
+end
+
+function InstructionsRace()
+        ShowMission(loc("Racer"),
+        	loc("A Hedgewars mini-game"),
+        	loc("Touch all waypoints as fast as you can!"),
+		2, 4000)
+end
 
 function onGameStart()
+	SendGameResultOff()
+	SendRankingStatsOff()
+        SendHealthStatsOff()
+	SendAchievementsStatsOff()
 
         roundN = 0
         lastRound = TotalRounds
-        RoundHasChanged = false -- true
+        RoundHasChanged = false
 
         for i = 0, (specialPointsCount-1) do
-                PlaceWayPoint(specialPointsX[i], specialPointsY[i])
+                PlaceWayPoint(specialPointsX[i], specialPointsY[i], false)
         end
 
         RebuildTeamInfo()
 
-        ShowMission     (
-                                loc("RACER"),
-                                loc("a Hedgewars mini-game"),
+        if specialPointsCount > 0 then
+                InstructionsRace()
+        else
+                InstructionsBuild()
+        end
 
-                                loc("Build a track and race.") .. "|" ..
-                                loc("Round Limit:") .. " " .. roundLimit .. "|" ..
+        SetAmmoTexts(amAirAttack, loc("Place waypoint"), loc("Racer tool"),
+                loc("Build an awesome race track by placing|waypoints which the hedgehogs have to|touch in any order to finish a round.") .. "|" ..
+		loc("Hedgehogs will start in the first waypoint.") .. "|" ..
+ 		loc("Cursor: Place waypoint") .. "|" ..
+ 		loc("Precise: Remove previous waypoint"))
 
-                                "", 4, 4000
-                                )
+        SetAmmoTexts(amSkip, loc("Finish waypoint placement"), loc("Racer tool"),
+                loc("Happy with your race track?|Then stop building and start racing!") .. "|" ..
+                loc("Or let the next player place waypoints|if less than 2 waypoints have been placed.") .. "|" ..
+                loc("Attack: Activate"))
 
         TryRepositionHogs()
 
 end
 
-function PlaceWayPoint(x,y)
+function PlaceWayPoint(x,y,placedByUser)
     if not racerActive then
         if wpCount == 0 or wpX[wpCount - 1] ~= x or wpY[wpCount - 1] ~= y then
 
             wpX[wpCount] = x
             wpY[wpCount] = y
-            wpCol[wpCount] = 0xffffffff
+            if wpCount == 0 then
+                wpCol[wpCount] = 0x80ff80ff
+            else
+                wpCol[wpCount] = 0xffffffff
+            end
             wpCirc[wpCount] = AddVisualGear(wpX[wpCount],wpY[wpCount],vgtCircle,0,true)
-                                                                                                                                            
+
             SetVisualGearValues(wpCirc[wpCount], wpX[wpCount], wpY[wpCount], 20, 100, 1, 10, 0, wpRad, 5, wpCol[wpCount])
 
             wpCount = wpCount + 1
 
-            AddCaption(loc("Waypoint placed.") .. " " .. loc("Available points remaining: ") .. (wpLimit-wpCount))
+            if placedByUser then
+                AddCaption(string.format(loc("Waypoint placed. Available points remaining: %d"), wpLimit-wpCount))
+            end
         end
     end
 end
 
+function onPrecise()
+    if not racerActive and CurrentHedgehog ~= nil and GetCurAmmoType() == amAirAttack then
+        DeletePreviousWayPoint()
+    end
+end
+
+function DeletePreviousWayPoint()
+    if wpCount > 0 then
+        wpCount = wpCount - 1
+        wpX[wpCount] = nil
+        wpY[wpCount] = nil
+        wpCol[wpCount] = nil
+        DeleteVisualGear(wpCirc[wpCount])
+        wpCirc[wpCount] = nil
+        AddCaption(string.format(loc("Waypoint removed. Available points: %d"), wpLimit-wpCount))
+    else
+        PlaySound(sndDenied)
+        AddCaption(loc("No waypoint to be removed!"))
+    end
+end
+
 function onSpecialPoint(x,y,flag)
-    specialPointsX[specialPointsCount] = x
-    specialPointsY[specialPointsCount] = y
-    specialPointsCount = specialPointsCount + 1
+    if flag == 99 then
+        fastX[fastCount] = x
+        fastY[fastCount] = y
+        fastCount = fastCount + 1
+    else
+        addHashData(x)
+        addHashData(y)
+        addHashData(flag)
+        specialPointsX[specialPointsCount] = x
+        specialPointsY[specialPointsCount] = y
+        specialPointsCount = specialPointsCount + 1
+    end
 end
 
 function onNewTurn()
@@ -562,6 +681,7 @@ function onNewTurn()
         TryRepositionHogs()
 
         racerActive = false
+        turnSkipped = false
 
         trackTime = 0
 
@@ -578,17 +698,24 @@ function onNewTurn()
 
         -- Handle Starting Stage of Game
         if (gameOver == false) and (gameBegun == false) then
-                if wpCount >= 3 then
+                if wpCount >= 2 then
                         gameBegun = true
                         roundNumber = 0
                         firstClan = GetHogClan(CurrentHedgehog)
-                        ShowMission(loc("RACER"),
-                        loc("GAME BEGUN!!!"),
-                        loc("Complete the track as fast as you can!"), 2, 4000)
+                        if specialPointsCount == 0 then
+                                InstructionsRace()
+                        end
+
+                        SetAmmoTexts(amSkip, nil, nil, nil)
                 else
-                        ShowMission(loc("RACER"),
-                        loc("NOT ENOUGH WAYPOINTS"),
-                        loc("Place more waypoints using the 'Air Attack' weapon."), 2, 4000)
+                        local infoString
+                        if wpLimit > 2 then
+                                infoString = string.format(loc("Place 2-%d waypoints using the waypoint placement tool."), wpLimit)
+                        else
+                                infoString = loc("Place 2 waypoints using the waypoint placement tool.")
+                        end
+                        ShowMission(loc("Racer"),
+                        loc("Waypoint placement phase"), infoString, 2, 4000)
                         AddAmmo(CurrentHedgehog, amAirAttack, 4000)
                         SetWeapon(amAirAttack)
                 end
@@ -617,16 +744,16 @@ function onGameTick20()
 
 
                         if TestRectForObstacle(x-20, y-20, x+20, y+20, true) then
-                                AddCaption(loc("Please place the way-point in the open, within the map boundaries."))
+                                AddCaption(loc("Please place the waypoint in the air, within the map boundaries"))
                                 PlaySound(sndDenied)
                         elseif (y > WaterLine-50) then
-                                AddCaption(loc("Please place the way-point further from the waterline."))
+                                AddCaption(loc("Please place the waypoint further away from the waterline"))
                                 PlaySound(sndDenied)
                         else
-                                PlaceWayPoint(x, y)
+                                PlaceWayPoint(x, y, true)
                                 if wpCount == wpLimit then
-                                        AddCaption(loc("Race complexity limit reached."))
-                                        DisableTumbler()
+                                        AddCaption(loc("Race complexity limit reached"))
+                                        EndTurn(true)
                                 end
                         end
                 else
@@ -649,7 +776,7 @@ function onGameTick20()
                                 trackTime = 0
 
                                 SetGearPosition(CurrentHedgehog, wpX[0], wpY[0])
-                                AddGear(GetX(CurrentHedgehog), GetY(CurrentHedgehog), gtGrenade, 0, 0, 0, 1)
+                                AddGear(GetX(CurrentHedgehog)+boostX, GetY(CurrentHedgehog)+boostY, gtGrenade, 0, 0, 0, 1)
                                 FollowGear(CurrentHedgehog)
 
                                 HideMission()
@@ -683,11 +810,7 @@ function onGameTick20()
 
                         if GameTime%100 == 0 then
 
-                if trackTime%1000 == 0 then
-                    AddCaption((trackTime/1000)..'.0',GetClanColor(GetHogClan(CurrentHedgehog)),capgrpMessage2)
-                else
-                    AddCaption(trackTime/1000,GetClanColor(GetHogClan(CurrentHedgehog)),capgrpMessage2)
-                end
+                                AddCaption(string.format(loc("Time: %.1fs"), (trackTime/1000)),GetClanColor(GetHogClan(CurrentHedgehog)),capgrpMessage2)
 
                                 if (CheckWaypoints() == true) then
                                         AdjustScores()
@@ -699,7 +822,7 @@ function onGameTick20()
                 end
 
                 -- if the player has expended his tunbling time, stop him tumbling
-                if TurnTimeLeft <= 20 then
+                if TurnTimeLeft <= 20 and not turnSkipped then
                         DisableTumbler()
                 end
 
@@ -712,7 +835,7 @@ function onGearResurrect(gear)
         AddVisualGear(GetX(gear), GetY(gear), vgtBigExplosion, 0, false)
 
         if gear == CurrentHedgehog then
-                DisableTumbler()
+                DisableTumbler(false)
         end
 
 end
@@ -743,17 +866,24 @@ end
 
 function onAttack()
     at = GetCurAmmoType()
-    
+
     usedWeapons[at] = 0
 end
 
+function onHogAttack(ammoType)
+    if ammoType == amSkip then
+        turnSkipped = true
+    end
+end
+
 function onAchievementsDeclaration()
     usedWeapons[amSkip] = nil
-    
+    usedWeapons[amExtraTime] = nil
+
     usedRope = usedWeapons[amRope] ~= nil
     usedPortal = usedWeapons[amPortalGun] ~= nil
     usedSaucer = usedWeapons[amJetpack] ~= nil
-    
+
     usedWeapons[amNothing] = nil
     usedWeapons[amRope] = nil
     usedWeapons[amPortalGun] = nil
@@ -775,11 +905,19 @@ function onAchievementsDeclaration()
         raceType = "mixed race"
     end
 
-    map = detectMap()
-    
+    map = detectMapWithDigest()
+
     for i = 0, (numTeams-1) do
-        if teamScore[i] < 100000 then
+        if teamScore[i] < 1000000 then
             DeclareAchievement(raceType, teamNameArr[i], map, teamScore[i])
         end
     end
+
+    if map ~= nil and fastCount > 0 then
+        StartGhostPoints(fastCount)
+
+        for i = 0, (fastCount - 1) do
+            DumpPoint(fastX[i], fastY[i])
+        end
+    end
 end
